import itertools
from collections import OrderedDict

from PyQt5.QtCore import pyqtSignal
from PyQt5.QtWidgets import QComboBox, QHeaderView
from PyQt5.QtGui import QFont
from PyQt5.QtWidgets import QApplication


class MyDict(OrderedDict):
    def __missing__(self, key):
        val = self[key] = MyDict()
        return val


class MyComboBox(QComboBox):
    popupAboutToBeShown = pyqtSignal()

    def showPopup(self):
        self.popupAboutToBeShown.emit()
        super(MyComboBox, self).showPopup()


def bold(item):
    bold = QFont()
    bold.setWeight(QFont.Bold)
    item.setFont(0, bold)


def resize_column(widget):
    widget.header().setSectionResizeMode(QHeaderView.ResizeToContents)


def peek(iterable):
    try:
        first = next(iterable)
    except StopIteration:
        return None
    return first, itertools.chain([first], iterable)


def icon_style(icon_name):
    style = QApplication.instance().style()
    return style.standardIcon(icon_name)



import os
import pendulum
from os.path import join

from PyQt5.QtCore import QObject, pyqtSlot

from Lupv.models.main import StudentRecords


class MainController(QObject):
    def __init__(self, main_model):
        super().__init__()
        self._record_path = ""
        self._main_model = main_model

        self._main_model.record_path_changed.connect(self.on_record_path_changed)

    def change_record_path(self, record_path):
        self._main_model.record_path = record_path

    @pyqtSlot(str)
    def on_record_path_changed(self, record_path):
        self._record_path = record_path

    def validate(self, path):
        """Check if path contain valid student directory.

        This is necessary because invalid directory will break `read_records` and
        make application crash.
        """
        dirs = os.listdir(path)
        invalid_student_dirs = []
        for d in dirs:
            if d != "lupv-notes":
                if not os.path.isdir(join(path, d, ".git")):
                    invalid_student_dirs.append(d)

        return invalid_student_dirs

    def relativize_datetime(self, datetime):
        """Convert datetime into its relative version."""
        dt = pendulum.instance(datetime)
        relative_time = dt.diff_for_humans()
        return relative_time

    def get_first_record_time(self, first_record_dt):
        """Take the first record time."""
        first_record_time = "{:%a, %d %b %Y, %H:%M:%S}".format(first_record_dt)
        first_record_relativetime = self.relativize_datetime(first_record_dt)
        return first_record_time, first_record_relativetime

    def get_last_record_time(self, last_record_dt):
        """Take the last record time."""
        last_record_time = "{:%a, %d %b %Y, %H:%M:%S}".format(last_record_dt)
        last_record_relativetime = self.relativize_datetime(last_record_dt)
        return last_record_time, last_record_relativetime

    def calc_work_duration(self, first_record_dt, last_record_dt):
        """Calculate duration between last and first record time."""
        work_duration = str(last_record_dt - first_record_dt)
        last_dt, first_dt = [
            pendulum.instance(x) for x in [last_record_dt, first_record_dt]
        ]
        work_relative_duration = (last_dt - first_dt).in_words(locale="en")
        return work_duration, work_relative_duration

    def populate_students_records(self):
        """Return list of student records."""
        students_records = self._main_model.students_records

        for student in students_records:
            name = student.name
            student_id = student.student_id
            total_records = len(student.records)

            first_record_dt, last_record_dt = [
                student.records[x].committed_datetime for x in [-1, 0]
            ]
            first_record_time, first_record_relativetime = self.get_first_record_time(
                first_record_dt
            )
            last_record_time, last_record_relativetime = self.get_last_record_time(
                last_record_dt
            )
            work_duration, work_relative_duration = self.calc_work_duration(
                first_record_dt, last_record_dt
            )

            student_records = StudentRecords(
                name,
                student_id,
                total_records,
                first_record_time,
                first_record_relativetime,
                last_record_time,
                last_record_relativetime,
                work_duration,
                work_relative_duration,
            )
            yield student_records



import yaml
import pathlib
import editdistance as edlib
from os.path import join
from collections import defaultdict

from PyQt5.QtCore import QObject

from Lupv.models.search import Suspects
from Lupv.models.search import StudentIp
from Lupv.models.search import StudentWindow
from Lupv.models.search import StudentEditdistances


class SearchController(QObject):
    def __init__(self, main_model, search_model, log_model):
        super().__init__()
        self._main_model = main_model
        self._search_model = search_model
        self._log_model = log_model

    def get_student_info(self, student_dir, datetime=None):
        """Extract student data from student directory name and format datetime."""
        date = None
        name, student_id = [str(student_dir).split("-")[x] for x in [0, 1]]
        if datetime:
            date = "{:%a, %d %b %Y, %H:%M:%S}".format(datetime)
        return name, student_id, date

    def populate_sample_filenames(self):
        """Return filenames sample."""
        # must take random student.
        # so that it can be use even before user visiting log view
        files = self._search_model.read_sample_files()
        return files

    def records_iterator(self):
        """Generator to iterate trough student records."""
        student_dirs = self._main_model.get_student_dirs()
        record_path = self._main_model.record_path

        for student in student_dirs:
            student_path = join(record_path, student)
            records = self._main_model.get_records(student_path)

            for record in records:
                self._log_model.current_student = student
                yield student, record

    def analyze_suspects(self, insertions_limit, filename):
        """Return students with exceeded the insertions limit."""
        suspects = []

        for student, record in self.records_iterator():
            if self._log_model.is_exists(filename, record.hexsha):
                insertions = record.stats.files[filename]["insertions"]

                if insertions > insertions_limit:
                    name, student_id, date = self.get_student_info(
                        student, record.committed_datetime
                    )

                    suspect = Suspects(name, student_id, filename, insertions, date)
                    suspects.append(suspect)

        return suspects

    def group_by_name(self, students):
        """Group students by name."""
        group = defaultdict(list)
        for student in students:
            student_nameid_key = "{}-{}".format(student.name, student.student_id)
            group[student_nameid_key].append(student)
        return group

    def get_suspects(self, insertions_limit, filename):
        """Return suspected students."""
        suspects = self.analyze_suspects(insertions_limit, filename)
        grouped_suspects = self.group_by_name(suspects)
        return grouped_suspects

    def get_student_ips(self):
        """Return student ips."""
        students_ip = []

        for student, record in self.records_iterator():
            auth_file = self._log_model.read_auth_info(record.hexsha)
            ip = auth_file[2]
            name, student_id, date = self.get_student_info(
                student, record.committed_datetime
            )

            student_ip = StudentIp(ip, name, student_id, date)
            students_ip.append(student_ip)

        return students_ip

    def group_by_ip(self, students):
        """Group students by ip."""
        group = defaultdict(list)
        for student in students:
            ip_key = "{}".format(student.ip)
            group[ip_key].append(student)
        return group

    def multigroup_child(self, grouped_students):
        """ Create multiple level of item groups:
        1. grouped_students (accepted data):
            - 123.0
              - ani
              - ani
              - ani
            - 111.0
              - budi
        2. extract each group items to temporary holder
            - holder[key:123.0] : ani,ani,ani
        3. arrange holder items by key (or similar things)
            - ani <-- parent level
                - ani
                - ani
        4. insert step three to new dict with grandparent keys
            - 123.0 <--- grand parent
              - ani <--- parent
                - ani <--- child
                - ani
            - 111.0
              - budi
              - ani
        `grouped_by_name = defaultdict(list)` placement is crucial, because it
        will be wiped to contain new group each iteration.
        """
        group = {}

        for parent_key in grouped_students.keys():
            # extract group items by parent key
            group_temp = grouped_students[parent_key]

            grouped_by_name = defaultdict(list)
            for student in group_temp:
                # arrange group item by similar identifier
                # e.g by name
                student_name_key = "{}-{}".format(student.name, student.student_id)
                grouped_by_name[student_name_key].append(student)

            # insert grouped parent-child item to it's grandparent
            group[parent_key] = grouped_by_name

        return group

    def get_student_ip_groups(self):
        """Return multi groped students ips."""
        students_ip = self.get_student_ips()
        grouped_by_ip = self.group_by_ip(students_ip)
        multi_group = self.multigroup_child(grouped_by_ip)
        return multi_group

    def idx_of_substring(self, mylist, substring):
        """Return index of substring in mylist."""
        for idx, string in enumerate(mylist):
            if substring in string:
                return idx

    def get_student_windows2(self, search_key):
        """Return windows name opened by students."""
        student_window = None
        student_windows = []

        for student, record in self.records_iterator():
            windows = self._log_model.read_all_windows(record.hexsha)
            windows_lower = [item.lower() for item in windows]

            found = self.idx_of_substring(windows_lower, search_key)
            if found:
                window_name = windows[found]
                student_name, student_id, date = self.get_student_info(
                    student, record.committed_datetime
                )

                student_window = StudentWindow(
                    window_name, student_name, student_id, date
                )
                # yield student_window
                student_windows.append(student_window)

        return student_windows

    def get_student_windows(self, search_key):
        ala = self.get_student_windows2(search_key)
        ali = self.group_by_name(ala)
        return ali

    #
    # Editdistance tab
    #
    def populate_student_dirs(self):
        """Return student directories."""
        student_dirs = self._main_model.get_student_dirs()
        return student_dirs

    def load_prev_editdistances(self, filename):
        """Load exported editdistances file."""
        editdistances = self._search_model.read_editdistances(filename)
        self._search_model.prev_editdistances = editdistances

    def get_prev_students(self):
        """Return loaded students name."""
        prev_editdistances = self._search_model.prev_editdistances
        prev_students = list(prev_editdistances.keys())
        return prev_students

    def get_prev_filename_sample(self):
        """Return loaded filename sample."""
        student_sample = self.get_prev_students()
        prev_editdistances = self._search_model.prev_editdistances
        sample_filename = prev_editdistances[student_sample[0]]["task_name"]
        return sample_filename

    def calc_prev_editdistances(self, student_name):
        """Return loaded editdistances value."""
        editdistances = self._search_model.prev_editdistances
        prev_records_ax = editdistances[student_name]["records_ax"]
        prev_editdistances_ax = editdistances[student_name]["editdistances_ax"]
        return prev_editdistances_ax, prev_records_ax

    def _get_student_records(self, student):
        """Return records of student."""
        record_path = self._main_model.record_path
        student_path = join(record_path, student)
        records = self._main_model.get_records(student_path)
        return records

    def calc_editdistances(self, student, filename):
        """Calculate student's editdistance and record axis."""
        records_count = 0
        records_ax = []
        editdistances_ax = []

        records = self._get_student_records(student)
        self._log_model.current_student = student
        last_file = self._log_model.read_file(filename, records[0].hexsha)

        for record in records:
            if self._log_model.is_exists(filename, record.hexsha):
                current_file = self._log_model.read_file(filename, record.hexsha)
                editdistance_value = edlib.eval(last_file, current_file)
                editdistances_ax.append(editdistance_value)

                records_count += 1
                records_ax.append(records_count)

        return editdistances_ax, records_ax

    def create_lupvnotes_dir(self):
        """Create lupv-notes directory."""
        record_path = self._main_model.record_path
        notes_path = join(record_path, "lupv-notes")
        pathlib.Path(notes_path).mkdir(parents=True, exist_ok=True)

    def construct_ed_graph_path(self, *args):
        """Return path for saving editdistance graph file."""
        record_path = self._main_model.record_path

        if len(args) == 1:
            graph_fmt = "{}.png".format(args[0])
        else:
            names = "_".join(args)
            graph_fmt = "{}.png".format(names)

        graph_path = join(record_path, "lupv-notes", graph_fmt)
        return graph_path

    def calc_all_editdistance(self, filename):
        """Calculate all students's editdistances."""
        for student, _ in self.records_iterator():
            editdistances_ax, records_ax = self.calc_editdistances(student, filename)

            name, student_id, _ = self.get_student_info(student)
            student_ed = StudentEditdistances(
                name, student_id, editdistances_ax, records_ax, filename
            )
            yield student_ed

    def construct_editdistance_path(self, task_name):
        """Return path for saving editdistance file."""
        record_path = self._main_model.record_path
        editdistance_file_path = join(
            record_path, "lupv-notes", "{}-editdistance.lup".format(task_name)
        )
        return editdistance_file_path

    def export_editdistance(self, students, save_path):
        """Export editdistances to file."""
        students_ed = {}
        for student in students:
            student_key = "{}-{}".format(student.name, student.student_id)
            students_ed[student_key] = dict(
                editdistances_ax=list(student.editdistances_ax),
                records_ax=list(student.records_ax),
                task_name=student.task_name,
            )
        self._search_model.write_editdistances(students_ed, save_path)



from PyQt5.QtWidgets import QMessageBox, QTreeWidgetItem, qApp
from PyQt5.QtGui import QBrush, QColor

from Lupv.standard.standard import resize_column


class LogView:
    def __init__(self, Ui, student_ctrl, controller):
        super().__init__()
        self._ui = Ui
        self._log_ctrl = student_ctrl
        self._main_ctrl = controller

        self._ui.log_tree.itemSelectionChanged.connect(self.log_selection_changed)

        self._ui.stats_check.setToolTip("Show/hide insertions deletions lines")
        self._ui.stats_check.clicked.connect(
            lambda: self.log_appearance_changed("stats")
        )

        self._ui.log_realdate_rbtn.setToolTip("Use Real DateTime format")
        self._ui.log_realdate_rbtn.toggled.connect(
            lambda: self.log_appearance_changed("dateformat")
        )
        self._ui.log_reldate_rbtn.setToolTip("Use Relative DateTime format")
        self._ui.log_reldate_rbtn.toggled.connect(
            lambda: self.log_appearance_changed("dateformat")
        )

        self._ui.diff_mode_rbtn.clicked.connect(self.log_selection_changed)
        self._ui.diff_mode_rbtn.setToolTip("Use diff mode for file contents")
        self._ui.show_mode_rbtn.clicked.connect(self.log_selection_changed)
        self._ui.show_mode_rbtn.setToolTip("Use show mode for file contents")
        self._ui.filename_combo.setToolTip("Filename to track")

    def toggle_spinner(self, toggle):
        """Show spinner icon while app working."""
        if toggle == "work":
            self._ui.spinner_stack.setCurrentIndex(1)
        else:
            self._ui.spinner_stack.setCurrentIndex(0)

    def clear_widgets(self):
        """Clear all widget contents."""
        self._ui.file_content_widget.clear()
        self._ui.log_tree.clear()
        self._ui.filename_combo.clear()
        self._ui.windows_tree.clear()

    def get_selected_sha(self):
        """Get SHA value from log_QTreeWidget."""
        sha = 0
        items = self._ui.log_tree.selectedItems()
        if items:
            sha = items[0].text(2)
        return sha

    def display_logs(self, complete=False):
        """Display log to log_QTreeWidget."""
        self.toggle_spinner("work")
        qApp.processEvents()

        self._ui.log_tree.clear()

        selected_file = self.get_selected_file()

        if complete:  # can't show stats without selected_file
            self._ui.log_tree.clear()
            for l in self._log_ctrl.populate_logs(selected_file):
                QTreeWidgetItem(
                    self._ui.log_tree,
                    [
                        str(l.relative_datetime),
                        str(l.datetime),
                        str(l.sha),
                        "{line}".format(
                            line="No record"
                            if l.add_stats == 0
                            else "{} Lines".format(l.add_stats)
                        ),
                        "{line}".format(
                            line="No record"
                            if l.del_stats == 0
                            else "{} Lines".format(l.del_stats)
                        ),
                    ],
                )
        else:
            for l in self._log_ctrl.populate_logs(selected_file):
                QTreeWidgetItem(
                    self._ui.log_tree,
                    [str(l.relative_datetime), str(l.datetime), str(l.sha)],
                )

        resize_column(self._ui.log_tree)
        self.toggle_spinner("ready")

    def display_file_content(self, sha):
        """Display diff to diff_QPlainTextEdit."""
        self._ui.file_content_widget.clear()

        if self._ui.diff_mode_rbtn.isChecked():
            mode = "diff"
        else:
            mode = "show"

        selected_file = self.get_selected_file()
        no_file_selected_msg = "No file selected, Please select one"
        no_file_rec_msg = 'No availibale record for "{}" in this period'.format(
            selected_file
        )
        if not selected_file:
            self._ui.file_content_widget.setPlainText(no_file_selected_msg)
        else:
            file_content = self._log_ctrl.populate_file_content(
                selected_file, sha, mode
            )
            if file_content == "" or file_content is None:
                self._ui.file_content_widget.setPlainText(no_file_rec_msg)
            else:
                if mode == "show":
                    self._ui.file_content_widget.setPlainText(file_content)
                else:
                    self._ui.file_content_widget.appendHtml(file_content)

    def display_filename(self):
        """Display file to file_QTreeWidget."""
        files = self._log_ctrl.populate_files()

        self._ui.filename_combo.clear()
        files.insert(0, "No File Selected")
        self._ui.filename_combo.addItems(files)

    def get_selected_file(self):
        """Return selected file in file_QTreeWidget."""
        filename = str(self._ui.filename_combo.currentText())
        if filename != "No File Selected":
            return filename

    def display_windows(self, sha):
        """Display all windows and focused window from self._ui.records."""
        self._ui.windows_tree.clear()

        focused_window = self._log_ctrl.populate_focused_window(sha)
        focused_row = QTreeWidgetItem(self._ui.windows_tree, [focused_window])

        focused_row.setForeground(0, QBrush(QColor("#41CD52")))
        windows = self._log_ctrl.populate_all_windows(sha)
        for window in windows:
            if window != focused_window:
                QTreeWidgetItem(self._ui.windows_tree, [window])

    def display_auth_info(self, sha):
        """Display auth information from self._ui.records."""
        auth_info = self._log_ctrl.populate_auth_info(sha)
        if auth_info:
            self._ui.name_lbl.setText(auth_info[0])
            self._ui.machine_lbl.setText(auth_info[1])
            self._ui.ip_lbl.setText(auth_info[2])

    def log_selection_changed(self):
        """Actions invoked when selection in log_QTreeWidget changed."""
        sha = self.get_selected_sha()
        if sha:
            self.display_file_content(sha)
            self.display_windows(sha)
            self.display_auth_info(sha)

    def log_appearance_changed(self, btn_name):
        """Actions when appearance setting changed."""
        if btn_name == "stats":
            self.toggle_stats()
        else:
            self.toggle_dateformat()
        resize_column(self._ui.log_tree)

    def toggle_stats(self):
        """Togggle insertions deletions columns visibility."""
        if self._ui.stats_check.isChecked():
            selected_file = self.get_selected_file()
            if not selected_file:
                QMessageBox.warning(None, "", "please choose a file")
                self._ui.stats_check.setChecked(False)
                return None

            self.display_logs(complete=True)
            for col_num in [3, 4]:
                self._ui.log_tree.showColumn(col_num)
        else:
            for col_num in [3, 4]:
                self._ui.log_tree.hideColumn(col_num)

    def toggle_dateformat(self):
        """Togggle dateformat columns visibility."""
        if self._ui.log_realdate_rbtn.isChecked():
            self._ui.log_tree.showColumn(1)
            self._ui.log_tree.hideColumn(0)
        else:
            self._ui.log_tree.showColumn(0)
            self._ui.log_tree.hideColumn(1)



import matplotlib.pyplot as plt
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas

from PyQt5.QtCore import QSize
from PyQt5.QtGui import QIcon
from PyQt5.QtWidgets import (
    QMessageBox,
    QTreeWidgetItem,
    QSizePolicy,
    qApp,
    QDialog,
    QPushButton,
    QHBoxLayout,
    QVBoxLayout,
    QComboBox,
    QFileDialog,
)

from Lupv.standard.standard import MyComboBox, bold, resize_column, peek


class SearchView:
    def __init__(self, Ui, search_ctrl):
        super().__init__()
        self._ui = Ui
        self._search_ctrl = search_ctrl

        self._ui.analyze_suspects_btn.clicked.connect(self.display_suspects)

        self.suspect_filename_combo = MyComboBox()
        self.suspect_filename_combo.setSizePolicy(
            QSizePolicy.MinimumExpanding, QSizePolicy.Preferred
        )
        self.suspect_filename_combo.popupAboutToBeShown.connect(
            self.display_suspect_filenames
        )
        self._ui.horizontalLayout_5.addWidget(self.suspect_filename_combo)

        self._ui.analyze_suspects_btn.setIcon(QIcon(":img/account-search.svg"))
        self._ui.analyze_suspects_btn.setIconSize(QSize(16, 16))
        self._ui.analyze_suspects_btn.setToolTip(
            "Search for Suspect.\nThis might take a while"
        )

        self._ui.windows_searchkey_widget.returnPressed.connect(
            self.display_student_windows
        )
        self._ui.windows_search_btn.clicked.connect(self.display_student_windows)
        self._ui.windows_search_btn.setIcon(QIcon(":img/account-search.svg"))
        self._ui.windows_search_btn.setIconSize(QSize(16, 16))
        self._ui.windows_search_btn.setToolTip(
            "Search window by name.\nThis might take a while"
        )

        self._ui.group_by_ip_btn.clicked.connect(self.display_student_ips)
        self._ui.group_by_ip_btn.setIcon(QIcon(":img/account-search.svg"))
        self._ui.group_by_ip_btn.setIconSize(QSize(16, 16))
        self._ui.group_by_ip_btn.setToolTip(
            "Group students by Ip Address.\nThis might take a while"
        )

        self._ui.load_editdistance_action.triggered.connect(self.load_editdistance_file)
        self._ui.export_editdistance_action.triggered.connect(
            self.show_ed_filename_dialog
        )
        self._ui.compare_editdistance_btn.clicked.connect(
            self.display_compared_editdistance
        )
        self._ui.compared_ed_savegraph_btn.clicked.connect(
            lambda: self.display_compared_editdistance(savep=True)
        )

        # editdistance page
        self.cur_student_name_combo = MyComboBox()
        self.cur_student_name_combo.setSizePolicy(
            QSizePolicy.Preferred, QSizePolicy.Preferred
        )
        self.cur_filename_combo = MyComboBox()
        self.cur_filename_combo.setSizePolicy(
            QSizePolicy.Preferred, QSizePolicy.Preferred
        )

        self.cur_student_name_combo.popupAboutToBeShown.connect(
            self.display_cur_students_name_ed
        )
        self.cur_filename_combo.popupAboutToBeShown.connect(
            self.display_cur_student_file_ed
        )
        self._ui.horizontalLayout_18.addWidget(self.cur_student_name_combo)
        self._ui.horizontalLayout_19.addWidget(self.cur_filename_combo)

        self.prev_student_name_combo = MyComboBox()
        self.prev_student_name_combo.setSizePolicy(
            QSizePolicy.Preferred, QSizePolicy.Preferred
        )
        self.prev_filename_combo = MyComboBox()
        self.prev_filename_combo.setSizePolicy(
            QSizePolicy.Preferred, QSizePolicy.Preferred
        )

        self.prev_student_name_combo.popupAboutToBeShown.connect(
            self.display_prev_students_name_ed
        )
        self.prev_filename_combo.popupAboutToBeShown.connect(
            self.display_prev_student_file_ed
        )

        self._ui.horizontalLayout_21.addWidget(self.prev_student_name_combo)
        self._ui.horizontalLayout_20.addWidget(self.prev_filename_combo)

        self.figure = plt.figure()
        self.canvas = FigureCanvas(self.figure)
        self._ui.verticalLayout_22.addWidget(self.canvas)

    def toggle_spinner(self, toggle):
        """Show spinner icon while app working."""
        if toggle == "work":
            self._ui.spinner_stack.setCurrentIndex(1)
        else:
            self._ui.spinner_stack.setCurrentIndex(0)

    def display_suspect_filenames(self):
        "Suggest filename to display suspect fieleds."
        files = self._search_ctrl.populate_sample_filenames()
        files.insert(0, "No File Selected")
        self.suspect_filename_combo.clear()
        self.suspect_filename_combo.addItems(files)

    def display_suspects(self):
        """Display suspected students."""
        self._ui.suspects_tree.clear()
        insertions_limit = str(self._ui.insertions_limit_spin.value())
        filename = self.suspect_filename_combo.currentText()

        msg = "please set limit higer than {}\nAbove 10 is recommended"
        if int(insertions_limit) == 0:
            QMessageBox.warning(None, "", msg.format(insertions_limit))
            return None
        if filename == "No File Selected" or not filename:
            QMessageBox.warning(None, "", "please select a file")
            return None

        self.toggle_spinner("work")
        qApp.processEvents()

        suspects = self._search_ctrl.get_suspects(int(insertions_limit), str(filename))

        if len(suspects.keys()) > 0:
            for student_name_key in suspects.keys():
                parent = QTreeWidgetItem(
                    self._ui.suspects_tree,
                    [
                        "{} [{}]".format(
                            student_name_key, len(suspects[student_name_key])
                        )
                    ],
                )
                bold(parent)
                for suspect in suspects[student_name_key]:
                    QTreeWidgetItem(
                        parent,
                        [
                            suspect.name,
                            str(suspect.student_id),
                            suspect.filename,
                            str(suspect.insertions),
                            str(suspect.date),
                        ],
                    )
        else:
            for col in range(1, 5):
                self._ui.suspects_tree.hideColumn(col)
            self._ui.suspects_tree.headerItem().setText(0, "")
            msg = 'No suspect found, for "{}" insertion limit'
            QTreeWidgetItem(self._ui.suspects_tree, [msg.format(insertions_limit)])

        resize_column(self._ui.suspects_tree)
        self.toggle_spinner("ready")

    def display_student_ips(self):
        """Display grouped students ip address."""
        self.toggle_spinner("work")
        qApp.processEvents()

        self._ui.group_by_ip_tree.clear()
        ip_groups = self._search_ctrl.get_student_ip_groups()

        if len(ip_groups.keys()) > 0:
            for ip_key in ip_groups.keys():
                parent = QTreeWidgetItem(
                    self._ui.group_by_ip_tree,
                    ["{} [{}]".format(ip_key, len(ip_groups[ip_key]))],
                )
                bold(parent)
                for student_key in ip_groups[ip_key].keys():
                    child = QTreeWidgetItem(
                        parent,
                        [
                            "{} [{}]".format(
                                student_key, len(ip_groups[ip_key][student_key])
                            )
                        ],
                    )
                    for student in ip_groups[ip_key][student_key]:
                        QTreeWidgetItem(
                            child,
                            [
                                str(student.ip),
                                student.name,
                                str(student.student_id),
                                student.date,
                            ],
                        )
        else:
            for col in range(1, 4):
                self._ui.group_by_ip_tree.hideColumn(col)
            self._ui.group_by_ip_tree.headerItem().setText(0, "")
            QTreeWidgetItem(self.group_by_ip_tree, ["No IP address found"])

        resize_column(self._ui.group_by_ip_tree)
        self.toggle_spinner("ready")

    def display_student_windows(self):
        """Display student's opened windows name."""
        self._ui.windows_search_tree.clear()
        search_key = self._ui.windows_searchkey_widget.text()
        if not search_key:
            QMessageBox.warning(None, "", "please supply the window name")
            return None  # magic line `break` alias.

        self.toggle_spinner("work")
        qApp.processEvents()
        student_windows = self._search_ctrl.get_student_windows(search_key)

        if student_windows:
            for student_name_key in student_windows.keys():
                parent = QTreeWidgetItem(
                    self._ui.windows_search_tree,
                    [
                        "{} [{}]".format(
                            student_name_key, len(student_windows[student_name_key])
                        )
                    ],
                )
                bold(parent)
                for window in student_windows[student_name_key]:
                    QTreeWidgetItem(
                        parent,
                        [
                            window.window_name,
                            window.name,
                            str(window.student_id),
                            window.date,
                        ],
                    )
        else:
            for col in range(1, 5):
                self._ui.windows_search_tree.hideColumn(col)
            self._ui.windows_search_tree.headerItem().setText(0, "")
            msg = 'No windows name for "{}" found'
            QTreeWidgetItem(self._ui.windows_search_tree, [msg.format(search_key)])

        resize_column(self._ui.windows_search_tree)
        self.toggle_spinner("ready")

    #
    # Editdistance tab
    #

    def display_cur_students_name_ed(self):
        """Display current student name in editdistance page."""
        students = self._search_ctrl.populate_student_dirs()
        self.cur_student_name_combo.clear()
        students.insert(0, "No Student Selected")
        self.cur_student_name_combo.addItems(students)

    def display_cur_student_file_ed(self):
        """Display current student filename in editdistance page."""
        filenames = self._search_ctrl.populate_sample_filenames()
        self.cur_filename_combo.clear()
        filenames.insert(0, "No File Selected")
        self.cur_filename_combo.addItems(filenames)

    def choosefile_dialog(self, caption):
        """Prompts dialog to choose record directory."""
        return QFileDialog.getOpenFileName(None, caption=caption)

    def load_editdistance_file(self):
        """Load exported editdistance file."""
        filename = self.choosefile_dialog("Select File")
        if not filename[0]:
            return None
        self._search_ctrl.load_prev_editdistances(filename[0])

    def display_prev_students_name_ed(self):
        """Display previous student name in editdistance page."""
        try:
            prev_students = self._search_ctrl.get_prev_students()
        except Exception:
            msg = "Please load editdistances file first"
            QMessageBox.warning(None, "File Not Loaded", msg)
            return None
        self.prev_student_name_combo.clear()
        prev_students.insert(0, "No Student Selected")
        self.prev_student_name_combo.addItems(prev_students)

    def display_prev_student_file_ed(self):
        """Display previous student filename in editdistance page."""
        try:
            sample_filename = self._search_ctrl.get_prev_filename_sample()
        except Exception:
            msg = "Please load editdistances file first"
            QMessageBox.warning(None, "File Not Loaded", msg)
            return None
        self.prev_filename_combo.clear()
        self.prev_filename_combo.addItem(sample_filename)

    def calc_prev_editdistances(self, student_name):
        """Calculate previous editdistances values."""
        prev_editdistances = self._search_model.prev_editdistances
        prev_records_ax = prev_editdistances[student_name]["records_ax"]
        prev_editdistances_ax = prev_editdistances[student_name]["editdistances_ax"]
        return prev_records_ax, prev_editdistances_ax

    def export_editdistance(self, filename):
        """Export current editdistances value to file."""
        self.toggle_spinner("work")
        qApp.processEvents()

        self._search_ctrl.create_lupvnotes_dir()
        students_ed = self._search_ctrl.calc_all_editdistance(filename)
        ed_path = self._search_ctrl.construct_editdistance_path(filename)
        self._search_ctrl.export_editdistance(students_ed, ed_path)
        msg = "Editdistance exported to {}"
        QMessageBox.information(None, "", msg.format(ed_path))

        self.toggle_spinner("ready")

    def show_ed_filename_dialog(self):
        """Prompt dialog for suspect parameter."""
        ed_filename_dlg = EdFilenameDialog()
        files = self._search_ctrl.populate_sample_filenames()
        ed_filename_dlg.editdistance_filename_combo.clear()
        ed_filename_dlg.editdistance_filename_combo.addItems(files)

        accepted = ed_filename_dlg.exec_()
        if accepted:
            filename = ed_filename_dlg.editdistance_filename_combo.currentText()
            if filename:
                self.export_editdistance(filename)
            else:
                QMessageBox.warning(None, "", "Please Choose filename")
                self.prompt_editdistance_dialog()

    def is_pair_filled(self, field1, field2):
        """Check if both fields filled."""
        if (field1 != "" and field1 != "No Student Selected") and (
            field2 != "" and field2 != "No File Selected"
        ):
            return True

    def display_compared_editdistance(self, savep=None):
        """Display editdistance graph."""
        cur_student_name = self.cur_student_name_combo.currentText()
        cur_filename = self.cur_filename_combo.currentText()
        prev_student_name = self.prev_student_name_combo.currentText()
        prev_filename = self.prev_filename_combo.currentText()

        cur_pair = self.is_pair_filled(cur_student_name, cur_filename)
        prev_pair = self.is_pair_filled(prev_student_name, prev_filename)
        if (cur_pair is None) and (prev_pair is None):
            QMessageBox.warning(None, "", "Please fill one of the pairs")
            return None

        self.figure.clear()
        ax = self.figure.add_subplot(111)
        cur_fmt = ""
        prev_fmt = ""
        bridge_fmt = ""

        if cur_pair is not None:
            cur_editdistances_ax, cur_records_ax = self._search_ctrl.calc_editdistances(
                cur_student_name, cur_filename
            )
            ax.plot(cur_records_ax, cur_editdistances_ax, label=cur_student_name)
            cur_fmt = "{}".format(cur_student_name)
            image_path = self._search_ctrl.construct_ed_graph_path(cur_student_name)

        if prev_pair is not None:
            prev_editdistances_ax, prev_records_ax = self._search_ctrl.calc_prev_editdistances(
                prev_student_name
            )
            ax.plot(prev_records_ax, prev_editdistances_ax, label=prev_student_name)
            prev_fmt = "{}".format(prev_student_name)
            image_path = self._search_ctrl.construct_ed_graph_path(prev_student_name)

        if (cur_pair and prev_pair) is not None:
            bridge_fmt = " .. "
            image_path = self._search_ctrl.construct_ed_graph_path(
                cur_student_name, prev_student_name
            )

        ax.legend()
        plt.title(
            "{cur}{bridge}{prev}".format(
                cur="" if cur_fmt is None else cur_fmt,
                bridge="" if bridge_fmt is None else bridge_fmt,
                prev="" if prev_fmt is None else prev_fmt,
            )
        )
        plt.xlabel("Records count")
        plt.ylabel("Editdistance from final sumbission")

        self.canvas.draw()

        if savep:
            plt.savefig(image_path)
            QMessageBox.information(None, "", "Graph saved to {}".format(image_path))


class EdFilenameDialog(QDialog):
    def __init__(self):
        super().__init__()

        self.cancel_btn = QPushButton("Close")
        self.export_btn = QPushButton("Export")

        btn_layout = QHBoxLayout()
        btn_layout.addWidget(self.export_btn)
        btn_layout.addWidget(self.cancel_btn)

        self.editdistance_filename_combo = QComboBox()

        layout = QVBoxLayout()
        layout.addWidget(self.editdistance_filename_combo)
        layout.addLayout(btn_layout)
        self.resize(281, 82)

        self.cancel_btn.clicked.connect(self.close)
        self.export_btn.clicked.connect(self.accept)

        self.setLayout(layout)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/media/azzamsya/rhd/azzamsa/self-management/references/masterpiece/software/code/projects/lup/lupv/lupv/views/main.ui'
#
# Created by: PyQt5 UI code generator 5.11.3
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(1324, 927)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.verticalLayout_7 = QtWidgets.QVBoxLayout()
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.widget = QtWidgets.QWidget(self.centralwidget)
        self.widget.setObjectName("widget")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.widget)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.page1_btn = QtWidgets.QPushButton(self.widget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.page1_btn.sizePolicy().hasHeightForWidth())
        self.page1_btn.setSizePolicy(sizePolicy)
        self.page1_btn.setMinimumSize(QtCore.QSize(0, 50))
        self.page1_btn.setText("")
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap("../img/lup.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.page1_btn.setIcon(icon)
        self.page1_btn.setIconSize(QtCore.QSize(64, 64))
        self.page1_btn.setObjectName("page1_btn")
        self.verticalLayout.addWidget(self.page1_btn)
        self.page2_btn = QtWidgets.QPushButton(self.widget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.page2_btn.sizePolicy().hasHeightForWidth())
        self.page2_btn.setSizePolicy(sizePolicy)
        self.page2_btn.setMinimumSize(QtCore.QSize(0, 50))
        self.page2_btn.setText("")
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap("../img/history.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.page2_btn.setIcon(icon1)
        self.page2_btn.setIconSize(QtCore.QSize(64, 64))
        self.page2_btn.setObjectName("page2_btn")
        self.verticalLayout.addWidget(self.page2_btn)
        self.page3_btn = QtWidgets.QPushButton(self.widget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.page3_btn.sizePolicy().hasHeightForWidth())
        self.page3_btn.setSizePolicy(sizePolicy)
        self.page3_btn.setMinimumSize(QtCore.QSize(0, 50))
        self.page3_btn.setText("")
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap("../img/search.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.page3_btn.setIcon(icon2)
        self.page3_btn.setIconSize(QtCore.QSize(64, 56))
        self.page3_btn.setObjectName("page3_btn")
        self.verticalLayout.addWidget(self.page3_btn)
        self.verticalLayout_2.addLayout(self.verticalLayout)
        self.horizontalLayout_2.addWidget(self.widget)
        self.stackedWidget = QtWidgets.QStackedWidget(self.centralwidget)
        self.stackedWidget.setObjectName("stackedWidget")
        self.page = QtWidgets.QWidget()
        self.page.setObjectName("page")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.page)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.groupBox_2 = QtWidgets.QGroupBox(self.page)
        self.groupBox_2.setMinimumSize(QtCore.QSize(0, 50))
        self.groupBox_2.setObjectName("groupBox_2")
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout(self.groupBox_2)
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.main_reldate_rbtn = QtWidgets.QRadioButton(self.groupBox_2)
        self.main_reldate_rbtn.setObjectName("main_reldate_rbtn")
        self.horizontalLayout_8.addWidget(self.main_reldate_rbtn)
        self.main_realdate_rbtn = QtWidgets.QRadioButton(self.groupBox_2)
        self.main_realdate_rbtn.setObjectName("main_realdate_rbtn")
        self.horizontalLayout_8.addWidget(self.main_realdate_rbtn)
        self.horizontalLayout_9.addLayout(self.horizontalLayout_8)
        self.verticalLayout_3.addWidget(self.groupBox_2)
        self.main_table = QtWidgets.QTableWidget(self.page)
        self.main_table.setObjectName("main_table")
        self.main_table.setColumnCount(9)
        self.main_table.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.main_table.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.main_table.setHorizontalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.main_table.setHorizontalHeaderItem(2, item)
        item = QtWidgets.QTableWidgetItem()
        self.main_table.setHorizontalHeaderItem(3, item)
        item = QtWidgets.QTableWidgetItem()
        self.main_table.setHorizontalHeaderItem(4, item)
        item = QtWidgets.QTableWidgetItem()
        self.main_table.setHorizontalHeaderItem(5, item)
        item = QtWidgets.QTableWidgetItem()
        self.main_table.setHorizontalHeaderItem(6, item)
        item = QtWidgets.QTableWidgetItem()
        self.main_table.setHorizontalHeaderItem(7, item)
        item = QtWidgets.QTableWidgetItem()
        self.main_table.setHorizontalHeaderItem(8, item)
        self.verticalLayout_3.addWidget(self.main_table)
        self.stackedWidget.addWidget(self.page)
        self.page_3 = QtWidgets.QWidget()
        self.page_3.setObjectName("page_3")
        self.gridLayout = QtWidgets.QGridLayout(self.page_3)
        self.gridLayout.setObjectName("gridLayout")
        self.groupBox_3 = QtWidgets.QGroupBox(self.page_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_3.sizePolicy().hasHeightForWidth())
        self.groupBox_3.setSizePolicy(sizePolicy)
        self.groupBox_3.setObjectName("groupBox_3")
        self.verticalLayout_17 = QtWidgets.QVBoxLayout(self.groupBox_3)
        self.verticalLayout_17.setObjectName("verticalLayout_17")
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.groupBox_5 = QtWidgets.QGroupBox(self.groupBox_3)
        self.groupBox_5.setObjectName("groupBox_5")
        self.verticalLayout_15 = QtWidgets.QVBoxLayout(self.groupBox_5)
        self.verticalLayout_15.setObjectName("verticalLayout_15")
        self.verticalLayout_14 = QtWidgets.QVBoxLayout()
        self.verticalLayout_14.setObjectName("verticalLayout_14")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.show_mode_rbtn = QtWidgets.QRadioButton(self.groupBox_5)
        self.show_mode_rbtn.setObjectName("show_mode_rbtn")
        self.horizontalLayout_3.addWidget(self.show_mode_rbtn)
        self.diff_mode_rbtn = QtWidgets.QRadioButton(self.groupBox_5)
        self.diff_mode_rbtn.setObjectName("diff_mode_rbtn")
        self.horizontalLayout_3.addWidget(self.diff_mode_rbtn)
        self.verticalLayout_14.addLayout(self.horizontalLayout_3)
        self.horizontalLayout_15 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_15.setObjectName("horizontalLayout_15")
        self.label_4 = QtWidgets.QLabel(self.groupBox_5)
        self.label_4.setObjectName("label_4")
        self.horizontalLayout_15.addWidget(self.label_4)
        self.filename_combo = QtWidgets.QComboBox(self.groupBox_5)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.filename_combo.sizePolicy().hasHeightForWidth())
        self.filename_combo.setSizePolicy(sizePolicy)
        self.filename_combo.setObjectName("filename_combo")
        self.horizontalLayout_15.addWidget(self.filename_combo)
        self.verticalLayout_14.addLayout(self.horizontalLayout_15)
        self.verticalLayout_15.addLayout(self.verticalLayout_14)
        self.horizontalLayout_7.addWidget(self.groupBox_5)
        self.log_appearance_box = QtWidgets.QGroupBox(self.groupBox_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.log_appearance_box.sizePolicy().hasHeightForWidth())
        self.log_appearance_box.setSizePolicy(sizePolicy)
        self.log_appearance_box.setCheckable(False)
        self.log_appearance_box.setObjectName("log_appearance_box")
        self.verticalLayout_16 = QtWidgets.QVBoxLayout(self.log_appearance_box)
        self.verticalLayout_16.setObjectName("verticalLayout_16")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout()
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.horizontalLayout_10 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_10.setObjectName("horizontalLayout_10")
        self.log_realdate_rbtn = QtWidgets.QRadioButton(self.log_appearance_box)
        self.log_realdate_rbtn.setObjectName("log_realdate_rbtn")
        self.horizontalLayout_10.addWidget(self.log_realdate_rbtn)
        self.log_reldate_rbtn = QtWidgets.QRadioButton(self.log_appearance_box)
        self.log_reldate_rbtn.setObjectName("log_reldate_rbtn")
        self.horizontalLayout_10.addWidget(self.log_reldate_rbtn)
        self.verticalLayout_4.addLayout(self.horizontalLayout_10)
        self.stats_check = QtWidgets.QCheckBox(self.log_appearance_box)
        self.stats_check.setObjectName("stats_check")
        self.verticalLayout_4.addWidget(self.stats_check)
        self.verticalLayout_16.addLayout(self.verticalLayout_4)
        self.horizontalLayout_7.addWidget(self.log_appearance_box)
        self.verticalLayout_17.addLayout(self.horizontalLayout_7)
        self.gridLayout.addWidget(self.groupBox_3, 0, 0, 1, 1)
        self.splitter_2 = QtWidgets.QSplitter(self.page_3)
        self.splitter_2.setOrientation(QtCore.Qt.Horizontal)
        self.splitter_2.setObjectName("splitter_2")
        self.splitter = QtWidgets.QSplitter(self.splitter_2)
        self.splitter.setOrientation(QtCore.Qt.Vertical)
        self.splitter.setObjectName("splitter")
        self.groupBox = QtWidgets.QGroupBox(self.splitter)
        self.groupBox.setMinimumSize(QtCore.QSize(0, 100))
        self.groupBox.setMaximumSize(QtCore.QSize(16777215, 100))
        self.groupBox.setObjectName("groupBox")
        self.horizontalLayout_14 = QtWidgets.QHBoxLayout(self.groupBox)
        self.horizontalLayout_14.setObjectName("horizontalLayout_14")
        self.horizontalLayout_13 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_13.setObjectName("horizontalLayout_13")
        self.horizontalLayout_11 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_11.setObjectName("horizontalLayout_11")
        self.verticalLayout_19 = QtWidgets.QVBoxLayout()
        self.verticalLayout_19.setObjectName("verticalLayout_19")
        self.name_const_lbl = QtWidgets.QLabel(self.groupBox)
        self.name_const_lbl.setObjectName("name_const_lbl")
        self.verticalLayout_19.addWidget(self.name_const_lbl)
        self.machine_const_lbl = QtWidgets.QLabel(self.groupBox)
        self.machine_const_lbl.setObjectName("machine_const_lbl")
        self.verticalLayout_19.addWidget(self.machine_const_lbl)
        self.horizontalLayout_11.addLayout(self.verticalLayout_19)
        self.verticalLayout_18 = QtWidgets.QVBoxLayout()
        self.verticalLayout_18.setObjectName("verticalLayout_18")
        self.name_lbl = QtWidgets.QLabel(self.groupBox)
        self.name_lbl.setText("")
        self.name_lbl.setObjectName("name_lbl")
        self.verticalLayout_18.addWidget(self.name_lbl)
        self.machine_lbl = QtWidgets.QLabel(self.groupBox)
        self.machine_lbl.setText("")
        self.machine_lbl.setObjectName("machine_lbl")
        self.verticalLayout_18.addWidget(self.machine_lbl)
        self.horizontalLayout_11.addLayout(self.verticalLayout_18)
        self.horizontalLayout_13.addLayout(self.horizontalLayout_11)
        self.horizontalLayout_12 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_12.setObjectName("horizontalLayout_12")
        self.verticalLayout_20 = QtWidgets.QVBoxLayout()
        self.verticalLayout_20.setObjectName("verticalLayout_20")
        self.ip_const_lbl = QtWidgets.QLabel(self.groupBox)
        self.ip_const_lbl.setObjectName("ip_const_lbl")
        self.verticalLayout_20.addWidget(self.ip_const_lbl)
        self.label_10 = QtWidgets.QLabel(self.groupBox)
        self.label_10.setText("")
        self.label_10.setObjectName("label_10")
        self.verticalLayout_20.addWidget(self.label_10)
        self.horizontalLayout_12.addLayout(self.verticalLayout_20)
        self.verticalLayout_21 = QtWidgets.QVBoxLayout()
        self.verticalLayout_21.setObjectName("verticalLayout_21")
        self.ip_lbl = QtWidgets.QLabel(self.groupBox)
        self.ip_lbl.setText("")
        self.ip_lbl.setObjectName("ip_lbl")
        self.verticalLayout_21.addWidget(self.ip_lbl)
        self.label_11 = QtWidgets.QLabel(self.groupBox)
        self.label_11.setText("")
        self.label_11.setObjectName("label_11")
        self.verticalLayout_21.addWidget(self.label_11)
        self.horizontalLayout_12.addLayout(self.verticalLayout_21)
        self.horizontalLayout_13.addLayout(self.horizontalLayout_12)
        self.horizontalLayout_14.addLayout(self.horizontalLayout_13)
        self.windows_tree = QtWidgets.QTreeWidget(self.splitter)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.windows_tree.sizePolicy().hasHeightForWidth())
        self.windows_tree.setSizePolicy(sizePolicy)
        self.windows_tree.setMinimumSize(QtCore.QSize(0, 100))
        self.windows_tree.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.windows_tree.setObjectName("windows_tree")
        self.windows_tree.headerItem().setText(0, "WIndows")
        self.file_content_widget = QtWidgets.QPlainTextEdit(self.splitter)
        self.file_content_widget.setMinimumSize(QtCore.QSize(0, 400))
        self.file_content_widget.setObjectName("file_content_widget")
        self.log_tree = QtWidgets.QTreeWidget(self.splitter_2)
        self.log_tree.setObjectName("log_tree")
        self.gridLayout.addWidget(self.splitter_2, 1, 0, 1, 1)
        self.stackedWidget.addWidget(self.page_3)
        self.page_4 = QtWidgets.QWidget()
        self.page_4.setObjectName("page_4")
        self.verticalLayout_13 = QtWidgets.QVBoxLayout(self.page_4)
        self.verticalLayout_13.setObjectName("verticalLayout_13")
        self.tabWidget = QtWidgets.QTabWidget(self.page_4)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.tabWidget.sizePolicy().hasHeightForWidth())
        self.tabWidget.setSizePolicy(sizePolicy)
        self.tabWidget.setTabPosition(QtWidgets.QTabWidget.North)
        self.tabWidget.setElideMode(QtCore.Qt.ElideNone)
        self.tabWidget.setMovable(False)
        self.tabWidget.setObjectName("tabWidget")
        self.suspects_tab = QtWidgets.QWidget()
        self.suspects_tab.setObjectName("suspects_tab")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.suspects_tab)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.horizontalLayout_16 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_16.setObjectName("horizontalLayout_16")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.label = QtWidgets.QLabel(self.suspects_tab)
        self.label.setObjectName("label")
        self.horizontalLayout_4.addWidget(self.label)
        self.insertions_limit_spin = QtWidgets.QSpinBox(self.suspects_tab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.insertions_limit_spin.sizePolicy().hasHeightForWidth())
        self.insertions_limit_spin.setSizePolicy(sizePolicy)
        self.insertions_limit_spin.setMaximum(10000)
        self.insertions_limit_spin.setObjectName("insertions_limit_spin")
        self.horizontalLayout_4.addWidget(self.insertions_limit_spin)
        self.horizontalLayout_16.addLayout(self.horizontalLayout_4)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.label_2 = QtWidgets.QLabel(self.suspects_tab)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_5.addWidget(self.label_2)
        self.horizontalLayout_16.addLayout(self.horizontalLayout_5)
        self.analyze_suspects_btn = QtWidgets.QPushButton(self.suspects_tab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.analyze_suspects_btn.sizePolicy().hasHeightForWidth())
        self.analyze_suspects_btn.setSizePolicy(sizePolicy)
        self.analyze_suspects_btn.setObjectName("analyze_suspects_btn")
        self.horizontalLayout_16.addWidget(self.analyze_suspects_btn)
        self.verticalLayout_5.addLayout(self.horizontalLayout_16)
        self.suspects_tree = QtWidgets.QTreeWidget(self.suspects_tab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.suspects_tree.sizePolicy().hasHeightForWidth())
        self.suspects_tree.setSizePolicy(sizePolicy)
        self.suspects_tree.setAnimated(True)
        self.suspects_tree.setObjectName("suspects_tree")
        self.suspects_tree.header().setStretchLastSection(False)
        self.verticalLayout_5.addWidget(self.suspects_tree)
        self.tabWidget.addTab(self.suspects_tab, "")
        self.windows_tab = QtWidgets.QWidget()
        self.windows_tab.setObjectName("windows_tab")
        self.verticalLayout_10 = QtWidgets.QVBoxLayout(self.windows_tab)
        self.verticalLayout_10.setObjectName("verticalLayout_10")
        self.verticalLayout_9 = QtWidgets.QVBoxLayout()
        self.verticalLayout_9.setObjectName("verticalLayout_9")
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.label_3 = QtWidgets.QLabel(self.windows_tab)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout_6.addWidget(self.label_3)
        self.windows_searchkey_widget = QtWidgets.QLineEdit(self.windows_tab)
        self.windows_searchkey_widget.setText("")
        self.windows_searchkey_widget.setObjectName("windows_searchkey_widget")
        self.horizontalLayout_6.addWidget(self.windows_searchkey_widget)
        self.windows_search_btn = QtWidgets.QPushButton(self.windows_tab)
        self.windows_search_btn.setObjectName("windows_search_btn")
        self.horizontalLayout_6.addWidget(self.windows_search_btn)
        self.verticalLayout_9.addLayout(self.horizontalLayout_6)
        self.windows_search_tree = QtWidgets.QTreeWidget(self.windows_tab)
        self.windows_search_tree.setAnimated(True)
        self.windows_search_tree.setObjectName("windows_search_tree")
        self.windows_search_tree.headerItem().setText(1, "Name")
        self.verticalLayout_9.addWidget(self.windows_search_tree)
        self.verticalLayout_10.addLayout(self.verticalLayout_9)
        self.tabWidget.addTab(self.windows_tab, "")
        self.ip_tab = QtWidgets.QWidget()
        self.ip_tab.setObjectName("ip_tab")
        self.verticalLayout_12 = QtWidgets.QVBoxLayout(self.ip_tab)
        self.verticalLayout_12.setObjectName("verticalLayout_12")
        self.verticalLayout_11 = QtWidgets.QVBoxLayout()
        self.verticalLayout_11.setObjectName("verticalLayout_11")
        self.group_by_ip_btn = QtWidgets.QPushButton(self.ip_tab)
        self.group_by_ip_btn.setObjectName("group_by_ip_btn")
        self.verticalLayout_11.addWidget(self.group_by_ip_btn)
        self.group_by_ip_tree = QtWidgets.QTreeWidget(self.ip_tab)
        self.group_by_ip_tree.setAnimated(True)
        self.group_by_ip_tree.setObjectName("group_by_ip_tree")
        self.group_by_ip_tree.headerItem().setText(0, "IP")
        self.group_by_ip_tree.headerItem().setText(2, "Nim")
        self.verticalLayout_11.addWidget(self.group_by_ip_tree)
        self.verticalLayout_12.addLayout(self.verticalLayout_11)
        self.tabWidget.addTab(self.ip_tab, "")
        self.tab = QtWidgets.QWidget()
        self.tab.setObjectName("tab")
        self.verticalLayout_23 = QtWidgets.QVBoxLayout(self.tab)
        self.verticalLayout_23.setObjectName("verticalLayout_23")
        self.horizontalLayout_25 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_25.setObjectName("horizontalLayout_25")
        self.horizontalLayout_24 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_24.setObjectName("horizontalLayout_24")
        self.horizontalLayout_22 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_22.setObjectName("horizontalLayout_22")
        self.horizontalLayout_18 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_18.setObjectName("horizontalLayout_18")
        self.label_5 = QtWidgets.QLabel(self.tab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_5.sizePolicy().hasHeightForWidth())
        self.label_5.setSizePolicy(sizePolicy)
        self.label_5.setObjectName("label_5")
        self.horizontalLayout_18.addWidget(self.label_5)
        self.horizontalLayout_22.addLayout(self.horizontalLayout_18)
        self.horizontalLayout_19 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_19.setObjectName("horizontalLayout_19")
        self.label_8 = QtWidgets.QLabel(self.tab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_8.sizePolicy().hasHeightForWidth())
        self.label_8.setSizePolicy(sizePolicy)
        self.label_8.setObjectName("label_8")
        self.horizontalLayout_19.addWidget(self.label_8)
        self.horizontalLayout_22.addLayout(self.horizontalLayout_19)
        self.horizontalLayout_24.addLayout(self.horizontalLayout_22)
        self.horizontalLayout_23 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_23.setObjectName("horizontalLayout_23")
        self.horizontalLayout_21 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_21.setObjectName("horizontalLayout_21")
        self.label_7 = QtWidgets.QLabel(self.tab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_7.sizePolicy().hasHeightForWidth())
        self.label_7.setSizePolicy(sizePolicy)
        self.label_7.setObjectName("label_7")
        self.horizontalLayout_21.addWidget(self.label_7)
        self.horizontalLayout_23.addLayout(self.horizontalLayout_21)
        self.horizontalLayout_20 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_20.setObjectName("horizontalLayout_20")
        self.label_9 = QtWidgets.QLabel(self.tab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_9.sizePolicy().hasHeightForWidth())
        self.label_9.setSizePolicy(sizePolicy)
        self.label_9.setObjectName("label_9")
        self.horizontalLayout_20.addWidget(self.label_9)
        self.horizontalLayout_23.addLayout(self.horizontalLayout_20)
        self.horizontalLayout_24.addLayout(self.horizontalLayout_23)
        self.horizontalLayout_25.addLayout(self.horizontalLayout_24)
        self.compare_editdistance_btn = QtWidgets.QPushButton(self.tab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.compare_editdistance_btn.sizePolicy().hasHeightForWidth())
        self.compare_editdistance_btn.setSizePolicy(sizePolicy)
        self.compare_editdistance_btn.setObjectName("compare_editdistance_btn")
        self.horizontalLayout_25.addWidget(self.compare_editdistance_btn)
        self.verticalLayout_23.addLayout(self.horizontalLayout_25)
        self.verticalLayout_22 = QtWidgets.QVBoxLayout()
        self.verticalLayout_22.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.verticalLayout_22.setObjectName("verticalLayout_22")
        self.verticalLayout_23.addLayout(self.verticalLayout_22)
        self.compared_ed_savegraph_btn = QtWidgets.QPushButton(self.tab)
        self.compared_ed_savegraph_btn.setObjectName("compared_ed_savegraph_btn")
        self.verticalLayout_23.addWidget(self.compared_ed_savegraph_btn)
        self.tabWidget.addTab(self.tab, "")
        self.verticalLayout_13.addWidget(self.tabWidget)
        self.stackedWidget.addWidget(self.page_4)
        self.horizontalLayout_2.addWidget(self.stackedWidget)
        self.verticalLayout_7.addLayout(self.horizontalLayout_2)
        self.spinner_stack = QtWidgets.QStackedWidget(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.spinner_stack.sizePolicy().hasHeightForWidth())
        self.spinner_stack.setSizePolicy(sizePolicy)
        self.spinner_stack.setMaximumSize(QtCore.QSize(16777215, 40))
        self.spinner_stack.setObjectName("spinner_stack")
        self.page_6 = QtWidgets.QWidget()
        self.page_6.setObjectName("page_6")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout(self.page_6)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.spinner_stack.addWidget(self.page_6)
        self.page_7 = QtWidgets.QWidget()
        self.page_7.setObjectName("page_7")
        self.horizontalLayout_17 = QtWidgets.QHBoxLayout(self.page_7)
        self.horizontalLayout_17.setObjectName("horizontalLayout_17")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.spinner_lbl = QtWidgets.QLabel(self.page_7)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.spinner_lbl.sizePolicy().hasHeightForWidth())
        self.spinner_lbl.setSizePolicy(sizePolicy)
        self.spinner_lbl.setText("")
        self.spinner_lbl.setPixmap(QtGui.QPixmap("../img/hourglass2.svg"))
        self.spinner_lbl.setObjectName("spinner_lbl")
        self.horizontalLayout.addWidget(self.spinner_lbl)
        self.label_6 = QtWidgets.QLabel(self.page_7)
        self.label_6.setObjectName("label_6")
        self.horizontalLayout.addWidget(self.label_6)
        self.horizontalLayout_17.addLayout(self.horizontalLayout)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_17.addItem(spacerItem)
        self.spinner_stack.addWidget(self.page_7)
        self.verticalLayout_7.addWidget(self.spinner_stack)
        self.verticalLayout_8.addLayout(self.verticalLayout_7)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 1324, 19))
        self.menubar.setObjectName("menubar")
        self.menuMenu = QtWidgets.QMenu(self.menubar)
        self.menuMenu.setObjectName("menuMenu")
        self.menuStorage = QtWidgets.QMenu(self.menubar)
        self.menuStorage.setObjectName("menuStorage")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.open_records_action = QtWidgets.QAction(MainWindow)
        self.open_records_action.setObjectName("open_records_action")
        self.quit_action = QtWidgets.QAction(MainWindow)
        self.quit_action.setObjectName("quit_action")
        self.light_theme_action = QtWidgets.QAction(MainWindow)
        self.light_theme_action.setObjectName("light_theme_action")
        self.dark_theme_action = QtWidgets.QAction(MainWindow)
        self.dark_theme_action.setObjectName("dark_theme_action")
        self.load_editdistance_action = QtWidgets.QAction(MainWindow)
        self.load_editdistance_action.setObjectName("load_editdistance_action")
        self.export_editdistance_action = QtWidgets.QAction(MainWindow)
        self.export_editdistance_action.setObjectName("export_editdistance_action")
        self.menuMenu.addAction(self.open_records_action)
        self.menuMenu.addAction(self.quit_action)
        self.menuStorage.addAction(self.load_editdistance_action)
        self.menuStorage.addAction(self.export_editdistance_action)
        self.menubar.addAction(self.menuMenu.menuAction())
        self.menubar.addAction(self.menuStorage.menuAction())

        self.retranslateUi(MainWindow)
        self.stackedWidget.setCurrentIndex(0)
        self.tabWidget.setCurrentIndex(0)
        self.spinner_stack.setCurrentIndex(1)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "Lupv"))
        self.groupBox_2.setTitle(_translate("MainWindow", "Appearance"))
        self.main_reldate_rbtn.setText(_translate("MainWindow", "Re&lative Date"))
        self.main_realdate_rbtn.setText(_translate("MainWindow", "Real &Date"))
        self.main_table.setSortingEnabled(True)
        item = self.main_table.horizontalHeaderItem(0)
        item.setText(_translate("MainWindow", "Name"))
        item = self.main_table.horizontalHeaderItem(1)
        item.setText(_translate("MainWindow", "Student ID"))
        item = self.main_table.horizontalHeaderItem(2)
        item.setText(_translate("MainWindow", "Total Records"))
        item = self.main_table.horizontalHeaderItem(3)
        item.setText(_translate("MainWindow", "First Record"))
        item = self.main_table.horizontalHeaderItem(4)
        item.setText(_translate("MainWindow", "First Record"))
        item = self.main_table.horizontalHeaderItem(5)
        item.setText(_translate("MainWindow", "Last Record"))
        item = self.main_table.horizontalHeaderItem(6)
        item.setText(_translate("MainWindow", "Last Record"))
        item = self.main_table.horizontalHeaderItem(7)
        item.setText(_translate("MainWindow", "Work Duration"))
        item = self.main_table.horizontalHeaderItem(8)
        item.setText(_translate("MainWindow", "Work Duration"))
        self.groupBox_3.setTitle(_translate("MainWindow", "Appearance"))
        self.groupBox_5.setTitle(_translate("MainWindow", "File Contens"))
        self.show_mode_rbtn.setText(_translate("MainWindow", "Show Mode"))
        self.diff_mode_rbtn.setText(_translate("MainWindow", "&Diff Mode"))
        self.label_4.setText(_translate("MainWindow", "Filename:"))
        self.log_appearance_box.setTitle(_translate("MainWindow", "Log Appearance"))
        self.log_realdate_rbtn.setText(_translate("MainWindow", "Rea&l Date"))
        self.log_reldate_rbtn.setText(_translate("MainWindow", "Re&lative Date"))
        self.stats_check.setText(_translate("MainWindow", "Line Insertion and Deletion"))
        self.groupBox.setTitle(_translate("MainWindow", "GroupBox"))
        self.name_const_lbl.setText(_translate("MainWindow", "<html><head/><body><p><span style=\" font-weight:600;\">Login Name:</span></p></body></html>"))
        self.machine_const_lbl.setText(_translate("MainWindow", "<html><head/><body><p><span style=\" font-weight:600;\">Machine name:</span></p></body></html>"))
        self.ip_const_lbl.setText(_translate("MainWindow", "<html><head/><body><p><span style=\" font-weight:600;\">IP Address:</span></p></body></html>"))
        self.log_tree.setSortingEnabled(True)
        self.log_tree.headerItem().setText(0, _translate("MainWindow", "Relative Date"))
        self.log_tree.headerItem().setText(1, _translate("MainWindow", "Date"))
        self.log_tree.headerItem().setText(2, _translate("MainWindow", "SHA"))
        self.log_tree.headerItem().setText(3, _translate("MainWindow", "Insertions"))
        self.log_tree.headerItem().setText(4, _translate("MainWindow", "Deletions"))
        self.label.setText(_translate("MainWindow", "Line Insetions Limit:"))
        self.label_2.setText(_translate("MainWindow", "Filename:"))
        self.analyze_suspects_btn.setText(_translate("MainWindow", "Search"))
        self.suspects_tree.headerItem().setText(0, _translate("MainWindow", "Name"))
        self.suspects_tree.headerItem().setText(1, _translate("MainWindow", "Nim"))
        self.suspects_tree.headerItem().setText(2, _translate("MainWindow", "Filename"))
        self.suspects_tree.headerItem().setText(3, _translate("MainWindow", "Line Insertions"))
        self.suspects_tree.headerItem().setText(4, _translate("MainWindow", "Date"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.suspects_tab), _translate("MainWindow", "Suspects"))
        self.label_3.setText(_translate("MainWindow", "Windows Name:"))
        self.windows_search_btn.setText(_translate("MainWindow", "Search"))
        self.windows_search_tree.headerItem().setText(0, _translate("MainWindow", "Windows Name"))
        self.windows_search_tree.headerItem().setText(2, _translate("MainWindow", "Nim"))
        self.windows_search_tree.headerItem().setText(3, _translate("MainWindow", "Record DateTime"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.windows_tab), _translate("MainWindow", "Windows"))
        self.group_by_ip_btn.setText(_translate("MainWindow", "Group by IP Address"))
        self.group_by_ip_tree.headerItem().setText(1, _translate("MainWindow", "Name"))
        self.group_by_ip_tree.headerItem().setText(3, _translate("MainWindow", "Record DateTime"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.ip_tab), _translate("MainWindow", "IP Address"))
        self.label_5.setText(_translate("MainWindow", "Student 1:"))
        self.label_8.setText(_translate("MainWindow", "Filename"))
        self.label_7.setText(_translate("MainWindow", "Student 2:"))
        self.label_9.setText(_translate("MainWindow", "Filename:"))
        self.compare_editdistance_btn.setText(_translate("MainWindow", "Compare"))
        self.compared_ed_savegraph_btn.setText(_translate("MainWindow", "Save Graph"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab), _translate("MainWindow", "EditDistance"))
        self.label_6.setText(_translate("MainWindow", "working . . ."))
        self.menuMenu.setTitle(_translate("MainWindow", "Me&nu"))
        self.menuStorage.setTitle(_translate("MainWindow", "Stora&ge"))
        self.open_records_action.setText(_translate("MainWindow", "&Open Records"))
        self.quit_action.setText(_translate("MainWindow", "&Quit"))
        self.light_theme_action.setText(_translate("MainWindow", "&Light"))
        self.dark_theme_action.setText(_translate("MainWindow", "&Dark"))
        self.load_editdistance_action.setText(_translate("MainWindow", "&Load Editdistance File"))
        self.export_editdistance_action.setText(_translate("MainWindow", "&Export Editdistance"))




import git
import os
from os.path import join

from PyQt5.QtCore import QObject, pyqtSignal


class LogModel(QObject):
    current_student_changed = pyqtSignal(str)
    record_path_changed = pyqtSignal(str)

    def __init__(self, main_model):
        super().__init__()
        self._record_path = ""
        self._current_student = ""
        self._student_path = ""
        self._student_repo = None
        self._student_records = None

        self._main_model = main_model
        self._record_path = self._main_model.record_path

        self.current_student_changed.connect(self.on_current_student_changed)

    @property
    def record_path(self):
        """Return the value of current student."""
        return self._record_path

    @record_path.setter
    def record_path(self, record_path):
        """Set the value of current student."""
        self._record_path = record_path
        self.record_path_changed.emit(record_path)

    @property
    def current_student(self):
        """Return the value of current student."""
        return self._current_student

    @current_student.setter
    def current_student(self, current_student):
        """Set the value of current student."""
        self._current_student = current_student
        self.current_student_changed.emit(current_student)

    @property
    def student_records(self):
        """Return the value of current student."""
        return self._student_records

    @student_records.setter
    def student_records(self, student_records):
        """Set the value of current student."""
        self._student_records = student_records

    @property
    def student_path(self):
        """Return the value of current student."""
        return self._student_path

    @student_path.setter
    def student_path(self, student_path):
        """Set the value of current student."""
        self._student_path = student_path

    @property
    def student_repo(self):
        """Return the value of current student."""
        return self._student_repo

    @student_repo.setter
    def student_repo(self, student_repo):
        """Set the value of current student."""
        self._student_repo = student_repo

    def on_current_student_changed(self):
        self.student_path = join(self._record_path, self._current_student)
        self.student_repo = git.Repo(self._student_path)
        self.student_records = self._main_model.get_records(self._student_path)

    def read_files(self):
        """Return files in student directory."""
        dirs = os.listdir(self._student_path)
        files = []
        for item in dirs:
            if os.path.isfile(join(self._student_path, item)):
                files.append(item)
        return files

    def read_focused_window(self, sha):
        """Read focused window from dotlup directory."""
        foc_win_path = join(".lup", "focused_window")
        focused_window = self._student_repo.git.show("{}:{}".format(sha, foc_win_path))
        return focused_window

    def read_auth_info(self, sha):
        """Read auth_info from dotlup directory."""
        auth_path = join(".lup", "auth_info")
        auth_file = self._student_repo.git.show("{}:{}".format(sha, auth_path))
        auth_info = auth_file.splitlines()
        return auth_info

    def read_all_windows(self, sha):
        """Read all windows from dotlup directories."""
        all_win_path = join(".lup", "all_windows")
        diff = self._student_repo.git.show("{}:{}".format(sha, all_win_path))
        windows = diff.splitlines()
        return windows

    def read_file(self, filename, sha):
        """Get content of current file state."""
        current_file = self._student_repo.git.show("{}:{}".format(sha, filename))
        return current_file

    def read_diff(self, filename, sha):
        """Get content of diff file."""
        diff = self._student_repo.git.show(sha, filename)
        return diff

    def is_exists(self, filename, sha):
        """Check if filename in current record exist."""
        files = self._student_repo.git.show("--pretty=" "", "--name-only", sha)
        if filename in files:
            return True


class Logs(QObject):
    def __init__(self, relative_datetime, datetime, sha, add_stats, del_stats):
        super().__init__()
        self.relative_datetime = relative_datetime
        self.datetime = datetime
        self.sha = sha
        self.add_stats = add_stats
        self.del_stats = del_stats



from PyQt5.QtWidgets import (
    QMainWindow,
    QFileDialog,
    QTableWidgetItem,
    QApplication,
    QMessageBox,
    QStyle,
    QLabel,
    qApp,
)

from PyQt5.QtCore import QSize, Qt, pyqtSlot
from PyQt5.QtGui import QKeySequence, QIcon, QPixmap

from Lupv.resources import images
from Lupv.standard.standard import icon_style
from Lupv.models.logs import LogModel
from Lupv.models.search import SearchModel
from Lupv.controllers.log import LogController
from Lupv.controllers.search import SearchController
from Lupv.views.main_ui import Ui_MainWindow
from Lupv.views.log import LogView
from Lupv.views.search import SearchView


class MainView(QMainWindow):
    def __init__(self, main_ctrl, main_model):
        super().__init__()
        self._main_ctrl = main_ctrl
        self._main_model = main_model
        self._record_path = ""
        self._ui = Ui_MainWindow()
        self._ui.setupUi(self)

        # Other page object
        self._log_model = LogModel(self._main_model)
        self._log_ctrl = LogController(self._main_ctrl, self._log_model)
        self._log_view = LogView(self._ui, self._log_ctrl, self._main_ctrl)

        self._search_model = SearchModel(self._main_model)
        self._search_ctrl = SearchController(
            self._main_model, self._search_model, self._log_model
        )
        self._search_view = SearchView(self._ui, self._search_ctrl)

        # Observers
        self._main_model.record_path_changed.connect(self.on_record_path_changed)

        # Sidebar
        self._ui.page1_btn.setIcon(QIcon(":img/lup.svg"))
        self._ui.page1_btn.setIconSize(QSize(69, 69))
        self._ui.page1_btn.setToolTip("Go to Main Window")

        self._ui.page2_btn.setIcon(QIcon(":img/history.svg"))
        self._ui.page2_btn.setIconSize(QSize(64, 64))
        self._ui.page2_btn.setToolTip("Go to Snapshot Window")

        self._ui.page3_btn.setIcon(QIcon(":img/search.svg"))
        self._ui.page3_btn.setIconSize(QSize(64, 64))
        self._ui.page3_btn.setToolTip("Go to Search Window")

        self._ui.page1_btn.clicked.connect(
            lambda: self._ui.stackedWidget.setCurrentIndex(0)
        )
        self._ui.page2_btn.clicked.connect(
            lambda: self._ui.stackedWidget.setCurrentIndex(1)
        )
        self._ui.page3_btn.clicked.connect(
            lambda: self._ui.stackedWidget.setCurrentIndex(2)
        )

        # Main View actions
        open_icon = icon_style(QStyle.SP_DialogOpenButton)
        self._ui.open_records_action.setIcon(open_icon)
        self._ui.open_records_action.triggered.connect(self.open_records)
        self._ui.open_records_action.setShortcut(QKeySequence("Ctrl+O"))
        self._ui.quit_action.triggered.connect(self.quit_app)
        self._ui.quit_action.setShortcut(QKeySequence("Ctrl+Q"))

        self._ui.main_table.clicked.connect(self.show_log_view)
        self._ui.main_table.setToolTip("Click me to analyze")

        self._ui.main_realdate_rbtn.setToolTip("Use Real DateTime format")
        self._ui.main_reldate_rbtn.setToolTip("Use Relative DateTime format")

        self._ui.main_realdate_rbtn.toggled.connect(
            lambda: self.change_table_appearance("real")
        )
        self._ui.main_reldate_rbtn.toggled.connect(
            lambda: self.change_table_appearance("rel")
        )

        hour_pixmap = QPixmap(":img/hourglass.svg")
        self._ui.spinner_lbl.setPixmap(hour_pixmap)

        self.welcome_lbl = QLabel()
        self.welcome_lbl.setText("Please open records to start analyzing")
        self.welcome_lbl.setStyleSheet("font-size: 20px;")
        self.welcome_lbl.setAlignment(Qt.AlignCenter)
        self._ui.verticalLayout.addWidget(self.welcome_lbl)
        self._ui.horizontalLayout_2.addWidget(self.welcome_lbl)

        # default
        self._ui.stackedWidget.setVisible(False)
        self._ui.widget.setVisible(False)
        self._ui.spinner_stack.setCurrentIndex(0)
        self._ui.load_editdistance_action.setEnabled(False)
        self._ui.export_editdistance_action.setEnabled(False)
        self._ui.tabWidget.tabBar().setExpanding(True)
        self._ui.stackedWidget.setCurrentIndex(0)

    def toggle_spinner(self, toggle):
        """Show spinner icon while app working."""
        if toggle == "work":
            self._ui.spinner_stack.setCurrentIndex(1)
        else:
            self._ui.spinner_stack.setCurrentIndex(0)

    def quit_app(self):
        """Quit application."""
        QApplication.quit()
        self.close()

    def choosepath_dialog(self, caption):
        """Prompts dialog to choose record directory."""
        options = QFileDialog.ShowDirsOnly | QFileDialog.DontResolveSymlinks
        return QFileDialog.getExistingDirectory(self, caption=caption, options=options)

    def is_valid(self, path):
        """Check if path is record path."""
        invalid_dirs = self._main_ctrl.validate(path)

        msg = "Not a valid Tasks directory\n"
        details = "\nContains invalid Task:\n{}".format("\n".join(invalid_dirs))

        if len(invalid_dirs) == 0:
            return True
        elif len(invalid_dirs) <= 10:
            QMessageBox.warning(self, "", msg + details)
        else:
            QMessageBox.warning(self, "", msg + "\n\nContains many invalid Tasks")

    def open_records(self):
        """Open records directory then display the record."""
        path = self.choosepath_dialog("Select Directory...")
        if not path:
            return None
        else:
            if not self.is_valid(path):
                return None

        self.toggle_spinner("work")
        qApp.processEvents()

        self._main_ctrl.change_record_path(path)
        self.display_records()

        self.toggle_spinner("ready")

    def display_records(self):
        """Populate records."""
        self._ui.main_table.setRowCount(0)

        for row_num, record in enumerate(self._main_ctrl.populate_students_records()):
            self._ui.main_table.insertRow(row_num)
            for col_num, record_item in enumerate(record.__dict__.items()):
                tbl_item = QTableWidgetItem(str(record_item[1]))
                self._ui.main_table.setItem(row_num, col_num, tbl_item)

        self._ui.main_table.setVisible(False)
        self._ui.main_table.verticalScrollBar().setValue(0)
        self._ui.main_table.resizeColumnsToContents()
        self._ui.main_table.setVisible(True)

        self.change_table_appearance("rel")  # default
        self._ui.main_reldate_rbtn.setChecked(True)
        self._ui.main_realdate_rbtn.setChecked(False)

        # hooks
        self.toggle_main_widgets()

    def change_table_appearance(self, btn_name):
        """Toggle columns visibility."""
        rel_col_hidden = True
        real_col_hidden = False
        rel_col_nums = [4, 6, 8]
        real_col_nums = [3, 5, 7]

        if btn_name == "rel":
            rel_col_hidden = False
            real_col_hidden = True

        for real_col, rel_col in zip(rel_col_nums, real_col_nums):
            self._ui.main_table.setColumnHidden(real_col, real_col_hidden)
            self._ui.main_table.setColumnHidden(rel_col, rel_col_hidden)

    def toggle_main_widgets(self):
        """Toggle main widget visibility."""
        self.welcome_lbl.setVisible(False)
        self._ui.stackedWidget.setVisible(True)
        self._ui.widget.setVisible(True)
        self._ui.load_editdistance_action.setEnabled(True)
        self._ui.export_editdistance_action.setEnabled(True)

    @pyqtSlot(str)
    def on_record_path_changed(self, path):
        self._log_ctrl.change_record_path(path)

    def get_selected_student(self):
        """Return selected student from main table"""
        name = self._ui.main_table.item(self._ui.main_table.currentRow(), 0).text()
        student_id = self._ui.main_table.item(
            self._ui.main_table.currentRow(), 1
        ).text()
        current_student = "{}-{}".format(name, student_id)
        return current_student

    def show_log_view(self):
        """Show Student Page and do initial things."""
        current_student = self.get_selected_student()
        self._log_ctrl.change_current_student(current_student)

        # default state
        for col in [0, 2, 3, 4]:
            self._ui.log_tree.hideColumn(col)
        self._ui.log_realdate_rbtn.setChecked(True)
        self._ui.stats_check.setChecked(False)
        self._ui.show_mode_rbtn.setChecked(True)

        # init functions
        self._log_view.clear_widgets()
        self._log_view.display_logs(False)
        self._log_view.display_filename()
        self._ui.stackedWidget.setCurrentIndex(1)



import sys

from PyQt5.QtWidgets import QApplication

from Lupv.models.main import MainModel
from Lupv.controllers.main import MainController
from Lupv.views.main import MainView


class Lupv(QApplication):
    def __init__(self, sys_argv):
        super().__init__(sys_argv)
        self.main_model = MainModel()
        self.main_ctrl = MainController(self.main_model)
        self.main_view = MainView(self.main_ctrl, self.main_model)
        self.main_view.show()


def main():
    lupv = Lupv(sys.argv)
    sys.exit(lupv.exec_())


if __name__ == "__main__":
    main()



import os
import yaml
from os.path import join

from PyQt5.QtCore import QObject


class SearchModel(QObject):
    def __init__(self, main_model):
        super().__init__()
        self._prev_editdistances = None
        self._main_model = main_model

    @property
    def prev_editdistances(self):
        """Return the value of current student."""
        return self._prev_editdistances

    @prev_editdistances.setter
    def prev_editdistances(self, prev_editdistances):
        """Set the value of current student."""
        self._prev_editdistances = prev_editdistances

    def read_sample_files(self):
        """Return files in student directory."""
        record_path = self._main_model.record_path
        sample_student = self._main_model.get_student_dirs()[0]
        sample_student_path = join(record_path, sample_student)
        dirs = os.listdir(sample_student_path)
        files = []

        for item in dirs:
            if os.path.isfile(join(sample_student_path, item)):
                files.append(item)
        return files

    def read_editdistances(self, filename):
        """Read exported editdistance file."""
        with open(filename, "r") as infile:
            editdistances = yaml.safe_load(infile)
        return editdistances

    def write_editdistances(self, students_ed, save_path):
        """Write exported value to file."""
        with open(save_path, "w") as outfile:
            yaml.dump(students_ed, outfile)


class Suspects(QObject):
    def __init__(self, name, student_id, filename, insertions, date):
        super().__init__()
        self.name = name
        self.student_id = student_id
        self.filename = filename
        self.insertions = insertions
        self.date = date


class StudentIp(QObject):
    def __init__(self, ip, name, student_id, date):
        super().__init__()
        self.ip = ip
        self.name = name
        self.student_id = student_id
        self.date = date


class StudentWindow(QObject):
    def __init__(self, window_name, name, student_id, date):
        super().__init__()
        self.window_name = window_name
        self.name = name
        self.student_id = student_id
        self.date = date


class StudentEditdistances(QObject):
    def __init__(self, name, student_id, editdistances_ax, records_ax, task_name):
        super().__init__()
        self.name = name
        self.student_id = student_id
        self.editdistances_ax = editdistances_ax
        self.records_ax = records_ax
        self.task_name = task_name



import pathlib
from setuptools import find_packages, setup

HERE = pathlib.Path(__file__).parent
README = (HERE / "README.md").read_text()
REQUIREMENTS = (HERE / "requirements.txt").read_text()

setup(
    name="Lupv",
    version="1.6.36",
    description="Lup viewer",
    long_description=README,
    long_description_content_type="text/markdown",
    author="azzamsa",
    author_email="azzam@azzamsa.com",
    url="https://gitlab.com/azzamsa/lupv",
    packages=find_packages(),
    include_package_data=True,
    license="GPLv3",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: Implementation :: CPython",
    ],
    install_requires=REQUIREMENTS,
    entry_points={"gui_scripts": ("lupv = Lupv.main:main")},
)



import editdistance as edlib

from PyQt5.QtCore import QObject

from Lupv.models.logs import Logs


class LogController(QObject):
    def __init__(self, main_ctrl, log_model):
        super().__init__()

        self._main_ctrl = main_ctrl
        self._log_model = log_model

    def change_current_student(self, current_student):
        self._log_model.current_student = current_student

    def change_record_path(self, record_path):
        """Change record path value."""
        self._log_model.record_path = record_path

    def populate_logs(self, selected_file=None):
        """Return student's log."""
        student_records = self._log_model.student_records

        for record in student_records:
            relative_time = self._main_ctrl.relativize_datetime(
                record.committed_datetime
            )
            time_format = "{:%a, %d %b %Y, %H:%M:%S}"
            time = time_format.format(record.committed_datetime)
            sha = record.hexsha
            insertions = 0
            deletions = 0

            if selected_file:
                if self._log_model.is_exists(selected_file, sha):
                    insertions = record.stats.files[selected_file]["insertions"]
                    deletions = record.stats.files[selected_file]["deletions"]

            log = Logs(relative_time, time, sha, insertions, deletions)
            yield log

    def populate_files(self):
        """Return list of files."""
        files = self._log_model.read_files()
        return files

    def populate_auth_info(self, sha):
        """Return student's auth information."""
        auth_info = self._log_model.read_auth_info(sha)
        return auth_info

    def populate_all_windows(self, sha):
        """Return all student's window name."""
        all_windows = self._log_model.read_all_windows(sha)
        return all_windows

    def populate_focused_window(self, sha):
        """Return focused window name."""
        focused = self._log_model.read_focused_window(sha)
        return focused

    def take_diff_body(self, diff):
        """Remove header from diff."""
        # FIXME use regex
        lines = diff.splitlines()
        diff_body = lines[11:]
        return "\n".join(diff_body)

    def wrap_with_html(self, diff):
        """Add html-css element into diff."""
        colored_diff = []
        css = "<span style='color:{color};white-space:pre;'>{line}</span>"
        lines = diff.splitlines()
        for line in lines:
            if line.startswith("-"):
                colored_line = css.format(color="red", line=line)
            elif line.startswith("+"):
                colored_line = css.format(color="green", line=line)
            else:
                colored_line = line
            colored_diff.append(colored_line)
        return "<br/>".join(colored_diff)

    def get_diff(self, filename, sha):
        dirty_diff = self._log_model.read_diff(filename, sha)
        diff_body = self.take_diff_body(dirty_diff)
        file_content = self.wrap_with_html(diff_body)
        return file_content

    def populate_file_content(self, filename, sha, mode="show"):
        "Return the content of current file according to specified mode."
        if self._log_model.is_exists(filename, sha):
            file_content = self._log_model.read_file(filename, sha)
            if mode == "diff":
                file_content = self.get_diff(filename, sha)

            return file_content

    def calc_editdistances(self, filename):
        """Calculate editdistance and record axis."""
        records_count = 0
        records_ax = []
        editdistances_ax = []

        student_records = self._log_model.student_records
        last_record_sha = student_records[0].hexsha
        last_file = self._log_model.show_file(filename, last_record_sha)

        for record in student_records:
            if self._log_model.is_exists(filename, record.hexsha):
                current_file = self._log_model.show_file(filename, record.hexsha)
                editdistance_value = edlib.eval(last_file, current_file)
                editdistances_ax.append(editdistance_value)

                records_count += 1
                records_ax.append(records_count)

        return editdistances_ax, records_ax

    def get_editdistance_values(self, filename):
        """Return value of editdistance axis and record axis."""
        records = self.get_student_records()
        editdistances_ax, records_ax = self._main_ctrl.calc_editdistances(
            filename, records, self.get_student_repo()
        )
        return editdistances_ax, records_ax



import git
import os
from os.path import join

from PyQt5.QtCore import QObject, pyqtSignal


class MainModel(QObject):
    record_path_changed = pyqtSignal(str)

    def __init__(self):
        super().__init__()
        self._record_path = ""
        self._students_records = None

        self.record_path_changed.connect(self.read_students_records)

    @property
    def record_path(self):
        """Return the value of record_path."""
        return self._record_path

    @record_path.setter
    def record_path(self, record_path):
        """Set the value of record_path."""
        self._record_path = record_path
        self.record_path_changed.emit(record_path)

    @property
    def students_records(self):
        """Return the value of record_path."""
        return self._students_records

    @students_records.setter
    def students_records(self, students_records):
        """Set the value of record_path."""
        self._students_records = students_records

    def get_student_dirs(self):
        "Return list of student directories"
        dirs = os.listdir(self._record_path)
        student_dirs = []
        for d in dirs:
            if d != "lupv-notes":
                if os.path.isdir(join(self._record_path, d, ".git")):
                    student_dirs.append(d)
        return student_dirs

    def get_records(self, student_path):
        """Return records from student directory."""
        student_repo = git.Repo(student_path)
        records = list(student_repo.iter_commits("master"))
        return records

    def read_students_records(self):
        """Read records from students directory."""
        student_dirs = self.get_student_dirs()
        students_records = []

        for student in student_dirs:
            student_path = join(self._record_path, student)
            records = self.get_records(student_path)

            name, student_id = [str(student).split("-")[x] for x in [0, 1]]

            student_records = StudentsRecords(name, student_id, records)
            students_records.append(student_records)

        self._students_records = students_records


class StudentsRecords(QObject):
    def __init__(self, name, student_id, records):
        super().__init__()
        self.name = name
        self.student_id = student_id
        self.records = records


class StudentRecords(QObject):
    def __init__(
        self,
        name,
        student_id,
        total_records,
        first_record_time,
        first_record_relativetime,
        last_record_time,
        last_record_relativetime,
        work_duration,
        work_relative_duration,
    ):
        super().__init__()
        self.name = name
        self.student_id = student_id
        self.total_records = total_records
        self.first_record_time = first_record_time
        self.first_record_relativetime = first_record_relativetime
        self.last_record_time = last_record_time
        self.last_record_relativetime = last_record_relativetime
        self.work_duration = work_duration
        self.work_relative_duration = work_relative_duration
